# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import unittest

from orquesta.expressions import base as expressions
from orquesta.expressions.functions import common as functions


class ExpressionEvaluatorTest(unittest.TestCase):

    def test_mix_types(self):
        expr = '<% ctx().foo %> and {{ ctx().foo }}'

        expected_errors = [
            {
                'type': None,
                'expression': expr,
                'message': 'Expression with multiple types is not supported.'
            }
        ]

        result = expressions.validate(expr)

        self.assertListEqual(
            expected_errors,
            result.get('errors', [])
        )

    def test_inspect_function_has_context_argument(self):
        self.assertTrue(expressions.func_has_ctx_arg(functions.ctx_))
        self.assertFalse(expressions.func_has_ctx_arg(functions.json_))
